%
%PLOT_CI   plots confidence intervals and patch between two confidence
%  interval lines. X is nx1 vector corresponding to a horizontal axis.
%  Y can be either nx1, nx2, or nx3 matrix.
%  If Y is nx1 vector, PLOT_CI plots just the main line. If Y is nx2, the
%  function assumes only two confidence intervals are to plot with the patch
%  enclosed between them. If Y is matrix of nx3, PLOT_CI plots, the main line,
%  two confidence interval lines, and also the patch between them.
%  The main line is specified by 1st column of matrix Y, whereas confidence
%  intervals are determined by 2nd and 3rd columns.
% 
%  PLOT_CI(...,parameter1,value1,parameter2,value2,...) allows setting
%  parameters for the main line, the patch, and the confidence interval lines,
%  such as line style, line width, color etc.
%  The function recognizes following parameters:
%    'MainLineWidth'
%    'MainLineStyle'
%    'MainLineColor'
%    'LineWidth'
%    'LineStyle'
%    'LineColor'
%    'PatchColor'
%    'PatchAlpha'
%    'AxesHandle'
%    'XScale'
%    'YScale'
% 
%  All parameters should be self-explanatory, however, all the 'MainLine*'
%  parameters set the main (middle) line of the graph. All 'Line*' 
%  parameters have the effect on confidence interval lines. If the parameter
%  'AxesHandle' is provided, PLOT_CI plots on axes specified by AxesHandle.
%  'XScale' and 'YScale' can change scale of an X or Y axis from 'linear' to
%  'log'. However, bear in mind that at the moment changing scale to log cases
%  resets 'PatchAlpha' to 1. It could not be solve at the moment.
% 
% H = PLOT_CI(...)
%   Returns structure of handles of the main line, confidence interval lines
%   and the patch.
% 
%   
%  EXAMPLES
%    % ----- Example 1 -----
%    % Plot a red patch between 2 dotted confidence interval curves with
%    % a midle solid black line.
%    x = -2:0.01:2;
%    x = x';
%    y1 = 0.5*x.^2;
%    y2 = 0.25*x.^2;
%    y3 = 0.9*x.^2;
%    plot_ci(x,[y1 y2 y3], 'PatchColor', 'r', 'PatchAlpha', 0.2, ...
%           'MainLineWidth', 2, 'MainLineStyle', '-', 'MainLineColor', 'b', ...
%           'LineWidth', 1.5, 'LineStyle','--', 'LineColor', 'k');
%    grid on;
%    % ----- end of example -----
%
% 
%    % ----- Example 2 -----
%    % Plot a yellow line with blue patch between 2 black solid lined confidence intervals
%    % and blue 50% transparent patch between them
%    y = [0.1489 0.2677 0.0718;...
%         0.2056 0.3032 0.1295;...
%         0.2733 0.3423 0.2120;...
%         0.3506 0.3903 0.3125;...
%         0.4347 0.4767 0.3934;...
%         0.5218 0.5997 0.4430;...
%         0.6079 0.7197 0.4862;...
%         0.6889 0.8200 0.5280;...
%         0.7618 0.8944 0.5689;...
%         0.8241 0.9437 0.6087];
%    N = 10;
%    x = linspace(40,95,N);
%    rgb_fill = [0.3047 0.6992 0.9102];
%    rgb_line = [0.9414 0.9414 0.3359];
%    figure;
%    plot_ci(x,y, 'PatchColor', rgb_fill, 'PatchAlpha', 0.5, ...
%           'MainLineWidth', 3, 'MainLineColor', rgb_line, ...
%           'LineWidth', 1, 'LineStyle','-', 'LineColor', 'k');
%    grid on;
%    % ----- end of example -----
%  
%
%    % ----- Example 3 -----
%    % This example is similar to previous, however, it shows that the
%    % graph can be built separately by holding an axes
%    y = [0.1489 0.2677 0.0718;...
%         0.2056 0.3032 0.1295;...
%         0.2733 0.3423 0.2120;...
%         0.3506 0.3903 0.3125;...
%         0.4347 0.4767 0.3934;...
%         0.5218 0.5997 0.4430;...
%         0.6079 0.7197 0.4862;...
%         0.6889 0.8200 0.5280;...
%         0.7618 0.8944 0.5689;...
%         0.8241 0.9437 0.6087];
%    N = 10;
%    x = linspace(40,95,N);
%    rgb_fill = [0.3047 0.6992 0.9102];
%    rgb_line = [0.9414 0.9414 0.3359];
%    figure;
%    plot_ci(x,y(:,1), 'MainLineWidth', 3);
%    hold on;
%    plot_ci(x,y(:,2:3), 'LineStyle',':', 'LineColor', 'k', ...
%           'PatchColor', rgb_fill, 'PatchAlpha', 0);
%    grid on;
%    hold off;
%    % ----- end of example -----
%
%    
%    % ----- Example 4 -----
%    % This example present the application of parameter AxesHandle
%    x = -2:0.01:2;
%    x = x';
%    y1 = 0.5*x.^2;
%    y2 = 0.25*x.^2;
%    y3 = 0.9*x.^2;
%    f = figure; ax = axes('Parent',f);
%    plot_ci(x,[y2 y3], 'PatchColor', 'r', 'PatchAlpha', 0.2, ...
%         'LineWidth', 1.5, 'LineStyle','--', 'LineColor', 'k', ...
% 		'AxesHandle',ax);
%    hold(ax,'all');
%    plot_ci(x,y1, 'MainLineWidth', 2, 'MainLineStyle', '-', 'MainLineColor', 'b', ...
%         'AxesHandle',ax);
%    grid on;
%    hold(ax,'off');
%    % ----- end of example -----
% 
% 
%    % ----- Example 5 -----
%    % Plot 2 graphs in one figure:
%    %    a red line with red patch of 10% transparency and
%    %    a green line with green patch of 10% transparency
%    % This example shows that 2 separate graphs can be drawn
%    % by by holding an axes
%    y1 = [0.1489 0.2677 0.0718;...
%         0.2056 0.3032 0.1295;...
%         0.2733 0.3423 0.2120;...
%         0.3506 0.3903 0.3125;...
%         0.4347 0.4767 0.3934;...
%         0.5218 0.5997 0.4430;...
%         0.6079 0.7197 0.4862;...
%         0.6889 0.8200 0.5280;...
%         0.7618 0.8944 0.5689;...
%         0.8241 0.9437 0.6087];
%    N = 10;
%    x = linspace(40,95,N);
%    figure;
%    plot_ci(x,y1, 'PatchColor', 'g', 'PatchAlpha', 0.1, ...
%           'MainLineWidth', 2, 'MainLineColor', 'g', ...
%           'LineWidth', 1, 'LineStyle','-', 'LineColor', 'k');
%    hold on
%    grid on;
%    
%    y2 = [0.0254 0.0640 0.0085; ...
%         0.0414 0.0785 0.0200; ...
%         0.0649 0.0964 0.0420; ...
%         0.0975 0.1215 0.0771; ...
%         0.1407 0.1705 0.1147; ...
%         0.1955 0.2572 0.1437; ...
%         0.2615 0.3717 0.1710; ...
%         0.3374 0.5016 0.1995; ...
%         0.4205 0.6322 0.2300; ...
%         0.5074 0.7494 0.2626];
%    plot_ci(x,y2, 'PatchColor', 'r', 'PatchAlpha', 0.1, ...
%           'MainLineWidth', 2, 'MainLineColor', 'r', ...
%           'LineWidth', 1, 'LineStyle','--', 'LineColor', 'k');
%    % ----- end of example -----

function Handle = plot_ci(X,Y,varargin)

error(nargchk(2, 32, nargin));

X = resize(X);
Y = resize(Y);

args = varargin;
N_varg = length(args);
if mod(N_varg,2) ~= 0
  error('Parameters must be input in pairs');
end

str_chk = true;
for n=1:2:N_varg
  str_chk = str_chk & ~isnumeric(args{n});
end
if ~str_chk
  error(['Parameter must be input in the form ''string'', ''value'', ' ...
  'although ''value'' may also be string.']);
end


%%% Figure Handle, Axes Handle and axis
param.axeshandle = [];
param.figurehandle = [];
param.axis.x = [];
param.axis.y = [];
for n=1:2:N_varg
  switch lower(args{n})
      case 'axeshandle'
        param.axeshandle = args{n+1};
      case 'figurehandle'
        param.figurehandle = args{n+1};
      case 'xscale'
        param.axis.x = args{n+1};
      case 'yscale';
        param.axis.y = args{n+1};
  end
end
param.name.main = {'mainlinewidth' 'mainlinestyle' 'mainlinecolor'};
param.name.line = {'linewidth' 'linestyle' 'linecolor'};
param.name.patch = {'patchcolor' 'patchalpha'};
param.name.marker = {'marker' 'markersize' 'markerfacecolor' 'markeredgecolor'};
param.name.handles = {'axeshandle' 'figurehandle'};
param.name.axis = {'xscale' 'yscale'};
param.value.main = {0 0 0};
param.value.line = {0 0 0};
param.value.patch = {[0 0 0] 0};
param.value.marker = {'' 0 [0 0 0] [0 0 0]};

%%% Checking Handles
if ~isempty(param.axeshandle)
  H_Axes = param.axeshandle;
  if ~isempty(param.figurehandle)
    I = param.axeshandle == allchild(param.figurehandle);
    I = logical(sum(I));
    if I
      set(0,'CurrentFigure',param.figurehandle);
    else
      disp('Handle of figure ignored due to not matching with handle of axes');
    end
  end
else
  H_Axes = newplot;
end

%%% Set Scale
if ~isempty(param.axis.x)
  set(H_Axes,'XScale',param.axis.x);
end
if ~isempty(param.axis.y)
  set(H_Axes,'YScale',param.axis.y);
end

%%% Setting parameters
if size(Y,2) == 1
  Hplot = plot(X,Y(:,1), 'Parent',H_Axes);
  for n=1:2:N_varg
    switch lower(args{n})
      case param.name.main{1}
        set(Hplot, 'LineWidth', args{n+1});
        param.value.main{1} = args{n+1};
      case param.name.main{2}
        set(Hplot, 'LineStyle', args{n+1});
        param.value.main{2} = args{n+1};
      case param.name.main{3}
        set(Hplot, 'Color', args{n+1});
        param.value.main{3} = args{n+1};
      case [param.name.handles param.name.axis]
      case [param.name.line param.name.patch param.name.marker]
        disp('Some parameters are not read');
      otherwise
        error('Unknown parameter. Please type ''help plot_patch'' for more information')
    end
  end
  Handle = Hplot;
elseif size(Y,2) == 2
  Hpatch = patch('Parent',H_Axes, ...
    'XData', [X(:,1);flipud(X(:,1))], ...
    'YData', [Y(:,1);flipud(Y(:,2))]);
  set(Hpatch, 'EdgeAlpha',0);
  H_hold = get(H_Axes,'NextPlot');
  set(H_Axes,'NextPlot','add');
  Hconf = plot(X, [Y(:,1) Y(:,2)], 'Parent',H_Axes);
  for n=1:2:N_varg
    switch lower(args{n})
      case param.name.line{1}
        set(Hconf(1), 'LineWidth', args{n+1});
        set(Hconf(2), 'LineWidth', args{n+1});
        param.value.line{1} = args{n+1};
      case param.name.line{2}
        set(Hconf(1), 'LineStyle', args{n+1});
        set(Hconf(2), 'LineStyle', args{n+1});
        param.value.line{2} = args{n+1};
      case param.name.line{3}
        set(Hconf(1), 'Color', args{n+1});
        set(Hconf(2), 'Color', args{n+1});
        param.value.line{3} = args{n+1};
      case param.name.patch{1}
        set(Hpatch, 'FaceColor', args{n+1});
        param.value.patch{1} = args{n+1};
      case param.name.patch{2}
        set(Hpatch, 'FaceAlpha', args{n+1});
        param.value.patch{2} = args{n+1};
      case param.name.marker{1}
        set(Hpatch, 'Marker', args{n+1});
        param.value.marker{1} = args{n+1};
      case param.name.marker{2}
        set(Hpatch, 'MarkerSize', args{n+1});
        param.value.marker{2} = args{n+1};
      case param.name.marker{3}
        set(Hpatch, 'MarkerFaceColor', args{n+1});
        param.value.marker{3} = args{n+1};
      case param.name.marker{4}
        set(Hpatch, 'MarkerEdgeColor', args{n+1});
        param.value.marker{4} = args{n+1};
      case [param.name.handles param.name.axis]
      case param.name.main
        disp('Some parameters are not read');
      otherwise
        error('Unknown parameter. Please type ''help plot_patch'' for more information')
    end
  end
  Handle.Confidence = Hconf;
  if param.value.patch{2} == 0
    delete(Hpatch);
  else
      Handle.Patch = Hpatch;
  end
  set(H_Axes, 'NextPlot',H_hold);
else
  Hpatch = patch('Parent', H_Axes, ...
    'XData', [X(:,1);flipud(X(:,1))], ...
    'YData', [Y(:,2);flipud(Y(:,3))]);
  set(Hpatch, 'EdgeAlpha',0);
  H_hold = get(H_Axes,'NextPlot');
  hold(H_Axes,'all');
  Hplot = plot(X,Y(:,1), 'Parent', H_Axes);
  Hconf = plot(X,[Y(:,2) Y(:,3)], 'Parent', H_Axes);
  for n=1:2:N_varg
    switch lower(args{n})
      case param.name.main{1}
        set(Hplot, 'LineWidth', args{n+1});
        param.value.main{1} = args{n+1};
      case param.name.main{2}
        set(Hplot, 'LineStyle', args{n+1});
        param.value.main{2} = args{n+1};
      case param.name.main{3}
        set(Hplot, 'Color', args{n+1});
        param.value.main{3} = args{n+1};
      case param.name.line{1}
        set(Hconf(1), 'LineWidth', args{n+1});
        set(Hconf(2), 'LineWidth', args{n+1});
        param.value.line{1} = args{n+1};
      case param.name.line{2}
        set(Hconf(1), 'LineStyle', args{n+1});
        set(Hconf(2), 'LineStyle', args{n+1});
        param.value.line{2} = args{n+1};
      case param.name.line{3}
        set(Hconf(1), 'Color', args{n+1});
        set(Hconf(2), 'Color', args{n+1});
        param.value.line{3} = args{n+1};
      case param.name.patch{1}
        set(Hpatch, 'FaceColor', args{n+1});
        param.value.patch{1} = args{n+1};
      case param.name.patch{2}
        set(Hpatch, 'FaceAlpha', args{n+1});
        param.value.patch{2} = args{n+1};
      case param.name.marker{1}
        set(Hpatch, 'Marker', args{n+1});
        param.value.marker{1} = args{n+1};
      case param.name.marker{2}
        set(Hpatch, 'MarkerSize', args{n+1});
        param.value.marker{2} = args{n+1};
      case param.name.marker{3}
        set(Hpatch, 'MarkerFaceColor', args{n+1});
        param.value.marker{3} = args{n+1};
      case param.name.marker{4}
        set(Hpatch, 'MarkerEdgeColor', args{n+1});
        param.value.marker{4} = args{n+1};
      case [param.name.handles param.name.axis]
      otherwise
        error('Unknown parameter. Please type ''help plot_patch'' for more information')
    end
  end
  Handle.Confidence = Hconf;
  Handle.Plot = Hplot;
  if param.value.patch{2} == 0
    delete(Hpatch);
  else
      Handle.Patch = Hpatch;
  end
  set(H_Axes, 'NextPlot',H_hold);
end

%%% functions
function X = resize(X, par)
if nargin < 2
  par = 1;
end
if ~isnumeric(par)
  if strcmpi(par,'horizontal')
    par = 2;
  elseif strcmpi(par,'vertical')
    par = 1;
  else
    error('wrong second parameter')
  end
end
if (par == 1)
  if size(X,1) < size(X,2)
    X = X';
  end
else
  if size(X,1) > size(X,2)
    X = X';
  end
end

% i = strmatch(lower(model), modelNames);
