% Set the parameters
addpath(genpath("./plot_ci"));
addpath(genpath("./mQUESTPlus"));
addpath(genpath("./matlab2tikz_code"));

% Set the parameters for thresholds
mean_threshold = 26;
variance_threshold = 36;
num_values = 10000;
lower_limit_threshold = 1;
upper_limit_threshold = 51;

% Set the parameters for slopes
mean_slope = 5.5;
variance_slope = 1.2;
lower_limit_slope = 1;
upper_limit_slope = 10;

% Set the parameters for lapses
mean_lapse = 0.02;
variance_lapse = 0.00002;
lower_limit_lapse = 0;
upper_limit_lapse = 0.04;

% Set the seed for reproducibility
rng(295636);

% Generate normally distributed random values for thresholds, slopes, and lapses
thresholds = normrnd(mean_threshold, sqrt(variance_threshold), [1, num_values]);
thresholds = max(lower_limit_threshold, min(upper_limit_threshold, thresholds));

slopes = normrnd(mean_slope, sqrt(variance_slope), [1, num_values]);
slopes = max(lower_limit_slope, min(upper_limit_slope, slopes));

lapses = normrnd(mean_lapse, sqrt(variance_lapse), [1, num_values]);
lapses = max(lower_limit_lapse, min(upper_limit_lapse, lapses));

% Set the search space
lower_limit = 1;
upper_limit = 51;
search_space = 1:51;

% Function to perform binary search for the threshold estimation
binarySearchThreshold = @(search_fun) relaxedBinarySearch(search_space, search_fun);

% Quest data
questData1 = qpInitialize('stimParamsDomainList', {1:1:51}, ...
    'psiParamsDomainList', {1:1:51, 1.0:0.1:10, 0:0.01:0.04}, 'qpPF', @qpPFNormal);

% Number of bootsrapping iterations
boots_itr = 1000;

% Check if GPU is available
gpuAvailable = (gpuDeviceCount() > 0);

% Preallocate arrays to store the results
thresholdsMLEQuest_all = cell(boots_itr, 20);
slopesMLEQuest_all = cell(boots_itr, 20);
lapsesMLEQuest_all = cell(boots_itr, 20);

% Parallel loop for bootstrap iterations
parfor itr = 1:boots_itr
    % Generate normally distributed random values for thresholds, slopes, and lapses
    thresholdBatch = normrnd(mean_threshold, sqrt(variance_threshold), [1, num_values]);
    thresholdBatch = max(lower_limit_threshold, min(upper_limit_threshold, thresholdBatch));
    
    slopeBatch = normrnd(mean_slope, sqrt(variance_slope), [1, num_values]);
    slopeBatch = max(lower_limit_slope, min(upper_limit_slope, slopeBatch));
    
    lapseBatch = normrnd(mean_lapse, sqrt(variance_lapse), [1, num_values]);
    lapseBatch = max(lower_limit_lapse, min(upper_limit_lapse, lapseBatch));
    
    if gpuAvailable
        % Transfer the data to the GPU
        thresholdBatch = gpuArray(thresholdBatch);
        slopeBatch = gpuArray(slopeBatch);
        lapseBatch = gpuArray(lapseBatch);
    end

    thresholdsMLEQuest = cell(1, 20);
    slopesMLEQuest = cell(1, 20);
    lapsesMLEQuest = cell(1, 20);

    for PCs_range = 30:30:600
        questData = questData1;

        for ntrials = 1:PCs_range
            observer_number = randi([1, num_values], 1);
            
            if gpuAvailable
                % Use gpuArray for computation on the GPU
                simulatedPsiParams = [thresholdBatch(observer_number), slopeBatch(observer_number), lapseBatch(observer_number)];
                simulatedObserverFun = @(x) qpSimulatedObserver(x, @qpPFNormal, gather(simulatedPsiParams));
            else
                % Use CPU for computation
                simulatedPsiParams = [thresholdBatch(observer_number), slopeBatch(observer_number), lapseBatch(observer_number)];
                simulatedObserverFun = @(x) qpSimulatedObserver(x, @qpPFNormal, simulatedPsiParams);
            end

            stim = qpQuery(questData);
            outcome = simulatedObserverFun(stim);
            questData = qpUpdate(questData, stim, outcome);
        end

        psiParamsIndex = qpListMaxArg(questData.posterior);
        psiParamsQuest = questData.psiParamsDomain(psiParamsIndex, :);
        psiParamsQuest(2) = psiParamsQuest(2) + 0.1;
        psiParamsFit = qpFit(questData.trialData, questData.qpPF, psiParamsQuest, questData.nOutcomes, ...
            'lowerBounds', [0 1 0], 'upperBounds', [51 10 0.04]);

        thresholdsMLEQuest{PCs_range/30} = psiParamsFit(1);
        slopesMLEQuest{PCs_range/30} = psiParamsFit(2);
        lapsesMLEQuest{PCs_range/30} = psiParamsFit(3);
    end

    thresholdsMLEQuest_all(itr, :) = thresholdsMLEQuest;
    slopesMLEQuest_all(itr, :) = slopesMLEQuest;
    lapsesMLEQuest_all(itr, :) = lapsesMLEQuest;
    disp(itr)
end

% Save the results
save result_collective_observer_600_1000_parallel